﻿using System;
using System.Diagnostics;
using System.Threading;
using System.Timers;
using Timer = System.Timers.Timer;

namespace CustomPerfCounterSample
{
    class Program
    {
        static void Main(string[] args)
        {
            PerfCounterSample.Instance.Start();

            for (int i = 0; i < 100; i++)
            {
                DoOperation();
            }
        }

        private static void DoOperation()
        {
            //do something here 
            PerfCounterSample.Instance.IncreaseOpCounter();
            Thread.Sleep(new Random().Next(2000));
        }
    }

    class PerfCounterSample
    {
        private const string CategoryName = "Sample Category";
        private const string CategoryHelp = "This is just a sample Category";
        private const string TotalOperationName = "Total Operation Counter";
        private const string TotalOperationHelp = "Count of Total Operation";
        private PerformanceCounter m_totalOperationCounter = null;
        private Timer m_Timer = new Timer();

        public static readonly PerfCounterSample Instance = new PerfCounterSample();

        private PerfCounterSample()
        {
            SetupCategory();
            CreatePerfCounter();
        }

        private void CreatePerfCounter()
        {
            m_totalOperationCounter = new PerformanceCounter(CategoryName, TotalOperationName, false);
        }

        private void SetupCategory()
        {
            if (PerformanceCounterCategory.Exists(CategoryName))
                PerformanceCounterCategory.Delete(CategoryName);
            CounterCreationDataCollection counters = GetCounterCollection();
            PerformanceCounterCategory.Create(CategoryName, CategoryHelp, counters);
        }

        private CounterCreationDataCollection GetCounterCollection()
        {
            CounterCreationDataCollection collection = new CounterCreationDataCollection();
            CounterCreationData totalOperation =
                new CounterCreationData(TotalOperationName, TotalOperationHelp, PerformanceCounterType.NumberOfItems32);
            collection.Add(totalOperation);
            return collection;
        }

        public void Start()
        {
            m_Timer.AutoReset = true;
            m_Timer.Interval = 1000.0;//1 Second
            m_Timer.Elapsed += new ElapsedEventHandler(Timer_Elapsed);
            m_Timer.Start();
        }

        void Timer_Elapsed(object sender, ElapsedEventArgs e)
        {
            //Write some date into perf counter
        }

        public void IncreaseOpCounter()
        {
            m_totalOperationCounter.Increment();
        }

    }
}
